@@ -1,6 +1,6 @@
 /*========================================================================*\
 
-Copyright (c) 1990-2003  Paul Vojta
+Copyright (c) 1990-2013  Paul Vojta
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -45,6 +45,11 @@ NOTE:
 #elif MOTIF
 #endif
 
+#if HAVE_XKB_BELL_EXT
+# include <X11/XKBlib.h>
+# define XBell(dpy, percent) XkbBell(dpy, mane.win, percent, (Atom) None)
+#endif
+
 #if	NeedVarargsPrototypes		/* this is for tell_oops */
 #include <stdarg.h>
 #else
@@ -661,7 +666,7 @@ change_font(n)
  *	Open a font file.
  */
 
-static	void
+void
 open_font_file(fontp)
 	struct font *fontp;
 {
@@ -849,6 +854,8 @@ prescan()
 	struct prescan_rec		pr;
 #endif
 
+	dvi_file_ready = False;
+
 	nextreportpage = scanned_page;
 	(void) lseek(fileno(dvi_file), page_info[scanned_page + 1].offset,
 	  SEEK_SET);
@@ -977,8 +984,13 @@ set_char P2C(wide_ubyte, cmd, wide_ubyte, ch)
 	    }
 	    if (g->addr == -1)
 		return ERRVAL;	/* previously flagged missing char */
-	    open_font_file(currinf.fontp);
-	    Fseek(currinf.fontp->file, g->addr, 0);
+#if FREETYPE
+	    if (currinf.fontp->ft == NULL)	/* if not freetype font */
+#endif
+	    {
+		open_font_file(currinf.fontp);
+		Fseek(currinf.fontp->file, g->addr, 0);
+	    }
 	    (*currinf.fontp->read_char)(currinf.fontp, ch);
 	    if (debug & DBG_BITMAP) print_char((ubyte) ch, g);
 	    currinf.fontp->timestamp = ++current_timestamp;
@@ -1129,6 +1141,88 @@ set_vf_char P2C(wide_ubyte, cmd, wide_ubyte, ch)
 }
 
 
+#if FREETYPE
+
+/*
+ *	set_ft_char() is used as a set_char routine to handle delayed loading
+ *	of freetype fonts.  See more details in ft.c.
+ */
+
+static void
+do_load_freetype_font()
+{
+	if (!load_ft_font(currinf.fontp)) {
+	    /* Revert to non-scalable font */
+	    struct ftfont *ftp;
+
+	    if (debug & DBG_OPEN)
+		Printf(
+		  "Font %s is not loadable; reverting to non-scalable font\n",
+		  currinf.fontp->fontname);
+
+	    ftp = currinf.fontp->ft;
+	    ftp->t1->bad = True;
+	    if (currinf.fontp == ftp->first_size) {
+		if (currinf.fontp->next_size == NULL) {
+		    /* if this is the only size of this font face */
+		    ftp->t1->ft = NULL;
+		    free(ftp);
+		}
+		else {
+		    struct font	*fontp2;
+
+		    ftp->first_size = fontp2 = currinf.fontp->next_size;
+		    /*
+		     * Opening the file might have succeeded at some other size,
+		     * so we need to transfer that information to the next
+		     * record in case it was put here.
+		     */
+		    fontp2->file = currinf.fontp->file;
+		    currinf.fontp->file = NULL;
+		    fontp2->filename = currinf.fontp->filename;
+		    currinf.fontp->filename = NULL;
+		    fontp2->timestamp = currinf.fontp->timestamp;
+		}
+	    }
+	    else {
+		struct font *fontp2;
+
+		fontp2 = ftp->first_size;
+		while (fontp2->next_size != currinf.fontp)
+		    fontp2 = fontp2->next_size;
+		fontp2->next_size = currinf.fontp->next_size;
+	    }
+	    currinf.fontp->ft = NULL;
+	    /* The virtual font machinery will take it from here.  */
+	    /* That will call load_font(), but it won't take us back to the */
+	    /* freetype font, because that font will have been marked bad.  */
+	    currinf.set_char_p = load_n_set_char;
+	}
+	else
+	    currinf.set_char_p = currinf.fontp->set_char_p = set_char;
+}
+
+# if !TEXXET
+long
+set_ft_char P1C(wide_ubyte, ch)
+# else
+void
+set_ft_char P2C(wide_ubyte, cmd, wide_ubyte, ch)
+# endif
+{
+	do_load_freetype_font();
+
+# if !TEXXET
+	return (*currinf.set_char_p)(ch);
+# else
+	(*currinf.set_char_p)(cmd, ch);
+	return;
+# endif
+}
+
+#endif /* FREETYPE */
+
+
 #ifndef	TEXXET
 static	long
 set_no_char P1C(wide_ubyte, ch)
@@ -1582,6 +1676,11 @@ geom_do_char(g_info, ch)
 	if (currinf.set_char_p == set_empty_char)
 	    return 0;	/* error; we'll catch it later */
 
+#if FREETYPE
+	if (currinf.set_char_p == set_ft_char)
+	    do_load_freetype_font();
+#endif
+
 	if (currinf.set_char_p == load_n_set_char) {
 	    if (ev_flags & EV_GE_NEWDOC)	/* if abort */
 		return 0;
@@ -1607,8 +1706,13 @@ geom_do_char(g_info, ch)
 		    return 0;	/* catch the error later */
 		if (g->addr == -1)
 		    return 0;	/* previously flagged missing char */
-		open_font_file(currinf.fontp);
-		Fseek(currinf.fontp->file, g->addr, 0);
+#if FREETYPE
+		if (currinf.fontp->ft == NULL)
+#endif
+		{
+		    open_font_file(currinf.fontp);
+		    Fseek(currinf.fontp->file, g->addr, 0);
+		}
 		(*currinf.fontp->read_char)(currinf.fontp, ch);
 		if (debug & DBG_BITMAP) print_char((ubyte) ch, g);
 		currinf.fontp->timestamp = ++current_timestamp;
@@ -2226,33 +2330,11 @@ static	struct findrec			search_tex	= {
 };
 
 
-#if !HAVE_ULLTOSTR
-
-#define	ulltostr xulltostr	/* prevents collision with header file def. */
-
-static	char *
-ulltostr(value, p)
-	unsigned long	value;
-	char		*p;
-{
-	do *--p = value % 10 + '0';
-	while ((value /= 10) != 0);
-
-	return p;
-}
-
-#endif
-
-
 static	void
 src_spawn_editor(parsed)
 	_Xconst struct src_parsed_special	*parsed;
 {
-#if HAVE_ULLTOSTR
-	char		scr_str[5 * sizeof(unsigned long long) / 2];
-#else
-	char		scr_str[5 * sizeof(unsigned long) / 2];
-#endif
+	char		scr_str[5 * sizeof(int) / 2];
 	_Xconst char	*filename2;
 	Boolean		found_filename	= False;
 	size_t		buffer_pos;
@@ -2378,18 +2460,26 @@ src_spawn_editor(parsed)
 			    len = 1;
 			    break;
 			case 'l':
-			    p1 = ulltostr(parsed->line,
-			      scr_str + sizeof scr_str);
-			    len = scr_str + sizeof scr_str - p1;
+			    p1 = scr_str;
+#if HAVE_VSNPRINTF
+			    len = snprintf(scr_str, sizeof scr_str, "%d",
+			      parsed->line);
+#else
+			    len = sprintf(scr_str, "%d", parsed->line);
+#endif
 			    break;
 			case 'c':
 			    if (parsed->col == 0) {
 				p += 2;
 				continue;
 			    }
-			    p1 = ulltostr(parsed->col,
-			      scr_str + sizeof scr_str);
-			    len = scr_str + sizeof scr_str - p1;
+			    p1 = scr_str;
+#if HAVE_VSNPRINTF
+			    len = snprintf(scr_str, sizeof scr_str, "%d",
+			      parsed->col);
+#else
+			    len = sprintf(scr_str, "%d", parsed->col);
+#endif
 			    break;
 			case 'f':
 			    p1 = filename2;
