@@ -1,6 +1,6 @@
 /*========================================================================*\
 
-Copyright (c) 1996-2001  Paul Vojta
+Copyright (c) 1996-2013  Paul Vojta
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -845,7 +845,8 @@ rd_cfg(paths)
 			    printf("Reading config file %s\n", ffline);
 			f = fopen(ffline, "r");
 			if (f == NULL) {
-			    if (FFDEBUG) perror(ffline);
+			    if (FFDEBUG)
+				printf("%s: %s\n", ffline, strerror(errno));
 			}
 			else {
 			    read_config_file(f);
@@ -1799,7 +1800,7 @@ init_quick_find(root, pos, atom, pos1)
 	f = xfopen(ffline, OPEN_MODE);
 	bcopy(tmpsav, ffline + pos, sizeof(tmpsav));
 	if (f == NULL) {
-	    if (FFDEBUG) perror("xfopen");
+	    if (FFDEBUG) printf("xfopen: %s\n", strerror(errno));
 	    return;
 	}
 
@@ -2100,7 +2101,7 @@ filltree(pos, treepp, atom, slashslash)
 
 	f = xopendir(ffline);
 	if (f == NULL) {
-	    if (FFDEBUG) perror(ffline);
+	    if (FFDEBUG) printf("%s: %s\n", ffline, strerror(errno));
 	    *treepp = NULL;
 	    return;
 	}
