@@ -1922,7 +1922,7 @@ myatopix(pp)
 	if (*cp >= 'a' && *cp <= 'z' && cp[1] >= 'a' && cp[1] <= 'z') {
 	    /* if units are present */
 	    if (p1 - p0 <= XtNumber(scr) - 3) {
-		Sprintf(scr, "%.*s%c%c", p1 - p0, p0, *cp, cp[1]);
+		Sprintf(scr, "%.*s%c%c", (int) (p1 - p0), p0, *cp, cp[1]);
 		value = atopix(scr, False);
 	    }
 	    else value = 0;
