@@ -1,6 +1,6 @@
 /*========================================================================*\
 
-Copyright (c) 1990-2004  Paul Vojta
+Copyright (c) 1990-2013  Paul Vojta
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -174,6 +174,11 @@ typedef	char		Bool3;		/* Yes/No/Maybe */
 # endif
 #endif
 
+#if FREETYPE
+# include <ft2build.h>
+# include FT_FREETYPE_H
+#endif
+
 #ifndef	NeedFunctionPrototypes
 #if	__STDC__
 #define	NeedFunctionPrototypes	1
@@ -540,14 +545,14 @@ struct font {
 	float fsize;			/* size information (dots per inch) */
 	int magstepval;			/* magstep number * two, or NOMAGSTP */
 	FILE *file;			/* open font file or NULL */
-	char *filename;			/* name of font file */
+	_Xconst char *filename;		/* name of font file */
 	long checksum;			/* checksum */
 	unsigned short timestamp;	/* for LRU management of fonts */
 	ubyte flags;			/* flags byte (see values below) */
 	ubyte maxchar;			/* largest character code */
 	double dimconv;			/* size conversion factor */
 	set_char_proc set_char_p;	/* proc used to set char */
-		/* these fields are used by (loaded) raster fonts */
+		/* these fields are used by (loaded) non-virtual fonts */
 	read_char_proc read_char;	/* function to read bitmap */
 	struct glyph *glyph;
 		/* these fields are used by (loaded) virtual fonts */
@@ -555,6 +560,13 @@ struct font {
 	struct tn *vf_chain;		/* ditto, if TeXnumber >= VFTABLELEN */
 	struct font *first_font;	/* first font defined */
 	struct macro *macro;
+#if FREETYPE
+		/* these fields are used by (loaded) FreeType fonts */
+	struct ftfont	*ft;		/* master record for font (all sizes) */
+	double		spsize;		/* scaled size of font in spell units */
+	FT_Size		size;
+	struct font	*next_size;	/* next font from same face */
+#endif
 		/* I suppose the above could be put into a union, but we */
 		/* wouldn't save all that much space. */
 };
@@ -613,6 +625,9 @@ extern	struct _resource {
 	_Xconst char	*src_pos;
 	Boolean		src_fork;
 	Boolean		_list_fonts;
+#if FREETYPE
+	Boolean		freetype;
+#endif
 	Boolean		reverse;
 	Boolean		_warn_spec;
 	Boolean		_hush_chars;
@@ -804,6 +819,8 @@ EXTERN	unsigned int	unshrunk_page_w, unshrunk_page_h;
 
 EXTERN	char		*dvi_name	INIT(NULL);	/* dvi file name */
 EXTERN	FILE		*dvi_file;		/* user's file */
+EXTERN	Boolean		dvi_file_ready	INIT(False);
+				/* done with font generation and prescanning */
 EXTERN	time_t		dvi_time;		/* last modification time */
 EXTERN	ino_t		dvi_inode;		/* used for source specials */
 EXTERN	unsigned char	*dvi_property;		/* for setting in window */
@@ -1032,6 +1049,61 @@ EXTERN	char	*ffline	INIT(NULL);	/* an array used by filefind to store */
 					/* Also used elsewhere.  */
 EXTERN	size_t	ffline_len INIT(0);	/* current length of ffline[] */
 
+
+#if FREETYPE || PS
+
+/*
+ *	AVL tree structures.
+ */
+
+#define	AVL_COMMON							\
+	_Xconst char	*key;		/* key */			\
+	int		key_len;	/* length of key */		\
+	int		bal;		/* AVL balancing information */	\
+	struct avl	*left;						\
+	struct avl	*right
+
+struct avl {		/* generic data structure */
+	AVL_COMMON;
+};
+
+	/* Data structure for Type 1 fonts -- contents of psfonts.map */
+struct avl_t1 {
+	AVL_COMMON;
+	_Xconst char	*psname;	/* PS name of font */
+	_Xconst char	*fontfile;	/* (short) name of pfa/pfb file */
+	_Xconst char	*encname;	/* (short) name of encoding file */
+	_Xconst char	*addinfo;	/* additional PS instructions */
+# if FREETYPE
+	Boolean		bad;		/* if later found to be unloadable */
+	struct ftfont	*ft;		/* pointer to FreeType record */
+# endif
+};
+
+
+# if FREETYPE
+
+struct ftfont {		/* info for FreeType font (Type 1 or TrueType) */
+	FT_Face		face;		/* NULL means not loaded yet */
+	struct font	*first_size;
+	struct avl_t1	*t1;
+	/* struct avl_tt *tt; */
+	struct FT_StreamRec_ stream;
+	struct avl_enc	*enc;		/* pointer to encoding record */
+	double		expn;		/* expansion factor */
+};
+
+# endif
+
+struct avl_enc {
+	AVL_COMMON;
+	Boolean		valid;
+	_Xconst char	*vec[256];
+};
+
+#endif /* FREETYPE || PS */
+
+
 /*
  *	Used by the geometry-scanning routines.
  *	It passes pointers to routines to be called at certain
@@ -1202,23 +1274,33 @@ extern	void	reload_dvi_file ARGS((void));
 extern	long	set_char ARGS((wide_ubyte));
 extern	long	load_n_set_char ARGS((wide_ubyte));
 extern	long	set_vf_char ARGS((wide_ubyte));
+# if FREETYPE
+extern	long	set_ft_char ARGS((wide_ubyte));
+# endif
 #else
 extern	void	set_char ARGS((wide_ubyte, wide_ubyte));
 extern	void	load_n_set_char ARGS((wide_ubyte, wide_ubyte));
 extern	void	set_vf_char ARGS((wide_ubyte, wide_ubyte));
+# if FREETYPE
+extern	long	set_ft_char ARGS((wide_ubyte, wide_ubyte));
+# endif
 #endif
+extern	void	open_font_file ARGS((struct font *));
 extern	void	prescan ARGS((void));
 extern	void	draw_page ARGS((void));
 extern	void	source_reverse_search ARGS((int, int));
 extern	void	source_special_show ARGS((wide_bool));
 extern	void	source_forward_search ARGS((_Xconst char *));
-#if	CFGFILE
+#if CFGFILE
 extern	void	readconfig ARGS((void));
 #endif
+#if FREETYPE || PS
+extern	FILE	*open_t1_font ARGS((struct avl_t1 *, _Xconst char **));
+extern	void	read_encoding ARGS((struct avl_enc *));
+#endif
 extern	void	init_font_open ARGS((void));
-extern	FILE	*font_open ARGS((_Xconst char *, char **, double, int *, int,
-			char **));
-#if	PS
+extern	Boolean	font_open ARGS((struct font *, char **, int *));
+#if PS
 extern	void	ps_clear_cache ARGS((void));
 extern	void	ps_destroy ARGS((void));
 #endif
@@ -1256,6 +1338,10 @@ extern	DIR	*xopendir ARGS((_Xconst char *));
 extern	_Xconst	struct passwd *ff_getpw ARGS((_Xconst char **, _Xconst char *));
 extern	unsigned long	num ARGS((FILE *, int));
 extern	long	snum ARGS((FILE *, int));
+#if FREETYPE || PS
+extern	struct avl *avladd ARGS((_Xconst char *, size_t, struct avl **,
+		  size_t));
+#endif
 extern	size_t	property_get_data ARGS((Window, Atom, unsigned char **,
 		  int (*x_get_property) (Display *, Window, Atom, long,
 		  long, Bool, Atom, Atom *, int *, unsigned long *,
@@ -1267,6 +1353,9 @@ extern	void	prep_fd ARGS((int, wide_bool));
 extern	void	read_PK_index ARGS((struct font *, wide_bool));
 extern	void	read_GF_index ARGS((struct font *, wide_bool));
 extern	void	read_VF_index ARGS((struct font *, wide_bool));
+#if FREETYPE
+extern	Boolean	load_ft_font ARGS((struct font *));
+#endif
 
 #if	PS
 extern	void	ps_init_paths ARGS((void));
